@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: a_io.c,v 1.11 1998/09/18 22:30:59 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: a_io.c,v 1.13 1998/11/03 18:00:06 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -392,9 +392,9 @@ parse_ircd()
 			** I can't think of any better way to handle this at
 			** the moment -kalt
 			*/
-			if (cldata[cl].state & A_IGNORE)
+			if (cldata[ncl].state & A_IGNORE)
 				break;
-			if (cldata[cl].state & A_LATE)
+			if (cldata[ncl].state & A_LATE)
 				/* pointless 99.9% of the time */
 				break;
 			if (cldata[ncl].authuser)
@@ -646,7 +646,7 @@ loop_io()
 			continue;
 #endif
 		    }
-		if (TST_READ_EVENT(cldata[i].rfd))
+		if (cldata[i].rfd > 0 && TST_READ_EVENT(cldata[i].rfd))
 		    {
 			int len;
  
@@ -672,7 +672,7 @@ loop_io()
 			    }
 			nfds--;
 		    }
-		else if (TST_WRITE_EVENT(cldata[i].wfd))
+		else if (cldata[i].wfd > 0 && TST_WRITE_EVENT(cldata[i].wfd))
 		    {
 			if (cldata[i].instance->mod->work(i) != 0)
 				next_io(i, cldata[i].instance);
