@@ -17,29 +17,18 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "struct.h"
-#include "common.h"
-#include "sys.h"
-#include "numeric.h"
-#include "h.h"
-
-#include <signal.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-#include "nameser.h"
-#include "resolv.h"
-#include "res.h"
+#include "os.h"
+#include "s_defines.h"
+#define RES_C
+#include "s_externs.h"
+#undef RES_C
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: res.c,v 1.9 1997/07/23 16:37:00 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: res.c,v 1.16 1998/09/09 22:36:18 kalt Exp $";
 #endif
 
 #undef	DEBUG	/* because there is a lot of debug code in here :-) */
 
-extern	int	errno, h_errno;
-extern	int	highest_fd;
-extern	aClient	*local[];
-
 static	char	hostbuf[HOSTLEN+1];
 static	char	dot[] = ".";
 static	int	incache = 0;
@@ -244,9 +233,12 @@ time_t	now;
 				switch (rptr->cinfo.flags)
 				    {
 				case ASYNC_CLIENT :
+#if defined(USE_IAUTH)
+					sendto_iauth("%d d", cptr->fd);
+#endif
 					ClearDNS(cptr);
-					if (!DoingAuth(cptr))
-						SetAccess(cptr);
+					if (!DoingAuth(cptr))	/* XAuth? */
+						SetAccess(cptr); /* unused */
 					break;
 				case ASYNC_CONNECT :
 					sendto_flag(SCH_ERROR,
@@ -385,7 +377,7 @@ Reg	ResRQ	*rptr;
 	char	hname[HOSTLEN+1];
 	int	len;
 
-	(void)strncpy(hname, name, sizeof(hname) - 1);
+	strncpyzt(hname, name, sizeof(hname));
 	len = strlen(hname);
 
 	if (rptr && !index(hname, '.') && ircd_res.options & RES_DEFNAMES)
@@ -527,7 +519,7 @@ HEADER	*hptr;
 	alias = hp->h_aliases;
 	while (*alias)
 		alias++;
-#ifdef	SVR4		/* brain damaged compiler (Solaris2) it seems */
+#if SOLARIS_2 && !defined(__GNUC__) /* brain damaged compiler it seems */
 	for (; hptr->qdcount > 0; hptr->qdcount--)
 #else
 	while (hptr->qdcount-- > 0)
@@ -666,7 +658,8 @@ char	*lp;
 	Reg	ResRQ	*rptr = NULL;
 	aCache	*cp = NULL;
 	struct	sockaddr_in	sin;
-	int	rc, a, len = sizeof(sin), max;
+	int	rc, a, max;
+	SOCK_LEN_TYPE len = sizeof(sin);
 
 	(void)alarm((unsigned)4);
 	rc = recvfrom(resfd, buf, sizeof(buf), 0, (SAP)&sin, &len);
@@ -1456,7 +1449,8 @@ int len;
 	char	*s, c;
 
 	for (s = name; (c = *s) && len; s++, len--)
-		if (isspace(c) || (c == 0x7) || (c == ':'))
+		if (isspace(c) || (c == 0x7) || (c == ':') ||
+		    (c == '*') || (c == '?'))
 			return -1;
 	return 0;
 }
