@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.47 1998/09/18 22:04:01 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.51 1998/11/12 11:44:52 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -1083,9 +1083,7 @@ char	*parv[];
 	    {
 		int	qlen = strlen(parv[2]);
 
-		if ((qlen < 4 || qlen < (int)strlen(parv[1]) ||
-		     index(parv[2]+1, '*') || index(parv[2]+1, '?')) &&
-		    IsServer(cptr) && check_link(cptr))
+		if (IsServer(cptr) && check_link(cptr) && !IsOper(sptr))
 		    {
 			sendto_one(sptr, rpl_str(RPL_TRYAGAIN, parv[0]),
 				   "LINKS");
@@ -1325,6 +1323,7 @@ char	*to;
 				   buf, cp->lseq, cp->lrecv,
 				   cp->ping / (cp->recv ? cp->recv : 1),
 				   tmp->pref);
+			sendto_flag(SCH_DEBUG, "%s: %d", buf, cp->seq);
 		    }
 	return;
 }
@@ -1344,7 +1343,7 @@ char	*parv[];
 
 	if (IsServer(cptr) &&
 	    (stat != 'd' && stat != 'p' && stat != 'q' && stat != 's' &&
-	     stat != 't' && stat != 'u' && stat != 'v') &&
+	     stat != 'u' && stat != 'v') &&
 	    !(stat == 'o' && IsOper(sptr)))
 	    {
 		if (check_link(cptr))
@@ -1611,8 +1610,8 @@ char	*parv[];
 	int i;
 
 	for (i = 0; msgtab[i].cmd; i++)
-		sendto_one(sptr,":%s NOTICE %s :%s",
-			   ME, parv[0], msgtab[i].cmd);
+	  sendto_one(sptr,":%s NOTICE %s :%s",
+		     ME, parv[0], msgtab[i].cmd);
 	return 2;
     }
 
@@ -1638,11 +1637,11 @@ char	*parv[];
 	aClient *acptr;
 
 	if (parc > 2)
-		if(hunt_server(cptr, sptr, ":%s LUSERS %s :%s", 2, parc, parv)
-				!= HUNTED_ISME)
+		if (hunt_server(cptr, sptr, ":%s LUSERS %s :%s", 2, parc, parv)
+		    != HUNTED_ISME)
 			return 3;
 
-	if (parc == 1)
+	if (parc == 1 || !MyConnect(sptr))
 	    {
 		sendto_one(sptr, rpl_str(RPL_LUSERCLIENT, parv[0]),
 			   istat.is_user[0] + istat.is_user[1],
@@ -2119,8 +2118,10 @@ char	*parv[];
 		case STAT_ME:
 			break;
 		case STAT_UNKNOWN:
-			sendto_one(sptr, rpl_str(RPL_TRACEUNKNOWN, parv[0]),
-				   class, name);
+			if (IsAnOper(sptr) || MyClient(sptr))
+				sendto_one(sptr,
+					   rpl_str(RPL_TRACEUNKNOWN, parv[0]),
+					   class, name);
 			break;
 		case STAT_CLIENT:
 			/* Only opers see users if there is a wildcard
@@ -2227,6 +2228,11 @@ char	*parv[];
 	struct	tm	*tm;
 #endif
 
+	if (check_link(cptr))
+	    {
+		sendto_one(sptr, rpl_str(RPL_TRYAGAIN, parv[0]), "MOTD");
+		return 5;
+	    }
 	if (hunt_server(cptr, sptr, ":%s MOTD :%s", 1,parc,parv)!=HUNTED_ISME)
 		return 5;
 #ifdef CACHED_MOTD
